SparseMatrixData

Type Alias SparseMatrixData 

pub type SparseMatrixData<'a, T> = HashMap<Shape, T>;
SparseMatrixData represents the datatype used to store information about non-zero values in a general matrix.

The keys are the index to the position in data, while the value is the value to be stored inside the matrix

Aliased Type

pub struct SparseMatrixData<'a, T> { /* private fields */ }